#include "Engine.h"
#include "Room.h"
#include "Meeting.h"
#include "ConnectionSettings.h"
#include "Configuration.h"
#include "DisplaySettings.h"
#include "CommunicationManager.h"
// #include "DeviceManager.h"
#include "Clock.h"
#include "ErrorMapper.h"
#include "UIManager.h"

#include <QApplication>
#include <QTimer>
#include <QList>
#include <QtDebug>

QTime Engine::endOfTheDay = QTime( 23, 59, 0, 0); // end of the day is 11:59pm
const int IDLE_TIME_MULTIPLIER = 60000; // Multiplies milliseconds to minutes

// Macro to help deleting objects. This could be global.
#define QT_DELETE(X) \
	if ( X != 0 ) \
	{ \
		delete X; \
		X = 0; \
	}


Engine::Engine() :
		iClock( 0 ), iConfiguration( 0 ), iCommunication( 0 ),
		iWindowManager( 0 ), iUIManager( 0 )
{
	qDebug() << "Engine::Engine()";
	iCommunicationFailed = false;
	iCurrentWeekFetched = false;
	
	initConfiguration();
	initDevice();
	initCommunication();
	initUserInterface();
	
	//initialize idle time counter
	iIdleTimeCounter = new QTimer();
	iIdleTimeCounter->setSingleShot( true);
	iIdleTimeCounter->setInterval(IDLE_TIME_MULTIPLIER * iConfiguration->displaySettings()->screensaver() );
	iIdleTimeCounter->start();

	// create application clock
	iClock = new Clock;
	connect( iClock, SIGNAL( tick( QDateTime ) ), this, SLOT( checkStatusOfAllRooms() ) );
	// connect( iClock, SIGNAL( tick( QDateTime ) ), iWindowManager, SLOT( distributeDateTimeInfo( QDateTime ) ) );

	// Create auto refresh timer
	iAutoRefresh = new QTimer;
	iAutoRefresh->setInterval(iConfiguration->connectionSettings()->refreshInterval() * 1000);
	iAutoRefresh->start();
	connect( iAutoRefresh, SIGNAL( timeout() ), iAutoRefresh, SLOT( start() ) );
	// connect( iAutoRefresh, SIGNAL( timeout() ), this, SLOT( fetchMeetings() ) );
	
	if( iDevice->currentOperationMode() == DeviceManager::KioskMode )
	{
		iWindowManager->setFullscreen();
	}

	connectSignals();
	
	// QTimer::singleShot( 0, this, SLOT( fetchMeetings() ) );

	// TODO: continue implementation
}

Engine::~Engine()
{
	qDebug() << "Engine::~Engine()";
	while ( !iMeetings.isEmpty() )
		delete iMeetings.takeFirst();
	
	if ( iIdleTimeCounter != 0 )
	{
		iIdleTimeCounter->stop();
		delete iIdleTimeCounter;
		iIdleTimeCounter = 0;
	}
	QT_DELETE( iClock );
	QT_DELETE( iDevice );

	QT_DELETE( iUIManager );
	QT_DELETE( iWindowManager );
}

void Engine::closeApplication()
{
	qDebug() << "Engine::closeApplication()";
	// closes application after 1 second
	QTimer::singleShot( 1000, QApplication::instance(), SLOT( quit() ));
}

Room* Engine::defaultRoom()
{
	qDebug() << "Engine::defaultRoom()";
	return iConfiguration->defaultRoom();
}

void Engine::checkStatusOfAllRooms()
{
//	qDebug() << "Engine::checkStatusOfAllRooms()";
	// iterate trough on the rooms
	for (int i = 0; i < iConfiguration->rooms().count(); i++)
	{
		// and check the status
		roomStatusInfoNeeded( iConfiguration->rooms().at(i) );
	}
}

int Engine::indexOfMeetingAt(Room *aRoom, QDateTime aAt)
{
//	qDebug() << "Engine::indexOfMeetingAt( Room *, QDateTime )";
	for ( int i = 0; i < iMeetings.count(); i++ )
	{
		// exchange server ensures that there is only one meeting in a room at a specified time
		if (aRoom->equals(iMeetings.at( i )->room() ) && iMeetings.at( i )->startsAt() <= aAt && iMeetings.at( i )->endsAt() >= aAt)
		{
			return i;
		}
	}
	return -1;
}

int Engine::indexOfMeetingAfter(Room *aRoom, QDateTime aAfter)
{
//	qDebug() << "Engine::indexOfMeetingAfter( Room *, QDateTime )";
	// seeks for the next meeting on the SAME DAY
	int min = -1;
	for (int i = 0; i < iMeetings.count(); i++)
	{
		// if the meeting is in the same room, on the same day but after the specified time
		if (aRoom->equals(iMeetings.at( i )->room() ) && iMeetings.at( i )->startsAt().date() == aAfter.date() && iMeetings.at( i )->startsAt() > aAfter)
		{
			// if there was not any meeting find yet or the previously found is a later one then the (i)th
			if (min == -1 || iMeetings.at( min )->startsAt() > iMeetings.at( i )->startsAt() )
			{
				min = i;
			}
		}
	}
	return min;
}

void Engine::roomStatusInfoNeeded(Room *aRoom)
{
//	qDebug() << "Engine::roomStatusInfoNeeded( Room * )";
	if ( aRoom == 0 )
	{
		return;
	}

	int indexOfCurrentMeeting = indexOfMeetingAt(aRoom, iClock->datetime() );
	int indexOfNextMeeting = indexOfMeetingAfter(aRoom, iClock->datetime() );

	// if there is no meeting, then status is Free; otherwise Busy
	Room::Status status = (indexOfCurrentMeeting == -1 ) ? Room::FreeStatus : Room::BusyStatus;
	// if room is Busy, then check end time, otherwise...
	QTime until = (status == Room::BusyStatus ) ? iMeetings.at( indexOfCurrentMeeting )->endsAt().time() :
	// ...if there is meeting following on the same day then check end time, otherwise end is the of the working day
	( ( indexOfNextMeeting != -1 ) ? iMeetings.at( indexOfNextMeeting )->startsAt().time() : Engine::endOfTheDay );

	//currently works only for deafult room
	if ( aRoom->equals( *(iCurrentRoom) ) )
	{
		emit roomStatusChanged( status, until );
	}
}

void Engine::fetchMeetingDetails( Meeting *aMeeting )
{
	qDebug() << "Engine::fetchMeetingDetails( Meeting* )";
	iCommunication->fetchMeetingDetails( *aMeeting );
}

void Engine::meetingsFetched( const QList<Meeting*> &aMeetings )
{
	qDebug() << "Engine::meetingsFetched( const QList<Meeting*> & )";
	// TODO: should check if this week's meetings were fetched
	if( iCommunicationFailed || !iCurrentWeekFetched )
	{
		iCurrentWeekFetched = true;
		iCommunicationFailed = false;
		iUIManager->connectionEstablished();
	}

	for ( int i = 0; i < iMeetings.count(); ++i ) 
	{
		Meeting* m = iMeetings.takeAt( i );
		delete m;
	}
	iMeetings.clear();
	for ( int i = 0; i < aMeetings.count(); ++i ) {
		Meeting* m = new Meeting( *( aMeetings.at( i ) ) );
		iMeetings.append( m );
	}

	// refresh room status info
	roomStatusInfoNeeded( iCurrentRoom /*defaultRoom()*/ );
}

void Engine::errorHandler( int aCode, const QString &aAddInfo )
{	
	if( aCode >= 100 && aCode < 150 )
	{
		iCommunicationFailed = true;
		if ( iUIManager != 0 ) iUIManager->connectionLost();
	}
	if ( iWindowManager != 0 )
	{
		iWindowManager->error( ErrorMapper::codeToString( aCode, aAddInfo ) );
	}
}

void Engine::fetchMeetings( const QDateTime &aFrom, const QDateTime &aUntil, const Room *aIn )
{
	qDebug()
			<< "Engine::fetchMeetings( const QDateTime &, const QDateTime &, const Room * )";
	iCommunication->fetchMeetings(aFrom, aUntil, *aIn);
}

void Engine::cancelFetchMeetingDetails()
{
	iCommunication->cancelFetchMeetingDetails();
}

void Engine::shownWeekChanged( QDate aFrom )
{
	qDebug() << "[Engine::shownWeekChanged] <Invoked>";
	QDateTime from( aFrom );
	QDateTime to( aFrom.addDays( 7 ), QTime( 23, 59 ) );
	qDebug() << "[Engine::shownWeekChanged] <From " << aFrom.toString( "d.m. h:mm" ) << " to " << to.toString( "d.m. h:mm" ) << ">";
	iCommunication->fetchMeetings( from, to, *iCurrentRoom/*defaultRoom()*/ );
}

void Engine::changeDeviceMode()
{
	connect( iDevice, SIGNAL( changeModeFailed() ), this, SLOT( changeModeFailed() ) );
	iAutoRefresh->stop(); // Stop the meeting update
	iDevice->changeMode();
}

void Engine::changeModeFailed()
{
	qDebug() << "Engine::progressBarCancelled()";
	iAutoRefresh->start(); //we start the metting updating
}

void Engine::initUserInterface()
{
	qDebug() << "[Engine::initUserInterface] <Invoked>";
	
	// Initialize the window manager and connect what ever signals can be connected
	iWindowManager = new WindowManager;
	// Create the UIManager which internally handles most of the UI actions
	iUIManager = new UIManager( this, iWindowManager );
	
	connect( iWindowManager, SIGNAL( eventDetected() ), this, SLOT( handleViewEvent() ) );
	connect( iWindowManager, SIGNAL( previousViewRestored() ), iUIManager, SLOT( previousViewRestored() ) );
	connect( iWindowManager, SIGNAL( dialogActivated() ), this, SLOT( dialogActivated() ) );
	connect( iWindowManager, SIGNAL( dialogDeactivated() ), this, SLOT( dialogDeactivated() ) );
	
	// Show the UI
	iWindowManager->setWindowState( Qt::WindowMaximized );
	iWindowManager->show();
	iUIManager->showMainView();
	
	// This triggers the meeting fetching
	iUIManager->currentRoomChanged( this->iCurrentRoom );
	
	qDebug() << "[Engine::initUserInterface] <Finished>";
}

void Engine::handleViewEvent()
{
	if ( iIdleTimeCounter != 0 && iIdleTimeCounter->isActive())
	{
		// Restart the idle time counter when view event is received
		iIdleTimeCounter->stop();
		iIdleTimeCounter->start();
	}
}

void Engine::initConfiguration()
{
	iConfiguration = Configuration::instance();
	if ( iConfiguration == 0 )
	{
		QTimer::singleShot( 0, this, SLOT( closeApplication() ) );
	}
	iCurrentRoom = iConfiguration->defaultRoom();
}

void Engine::connectSignals()
{
	// Connect engine objects signals to UIManager
	connect( iClock, SIGNAL( tick( QDateTime ) ), iUIManager, SLOT( updateTime( QDateTime ) ) );
	connect( iIdleTimeCounter, SIGNAL( timeout() ) , iUIManager, SLOT( roomStatusIndicatorRequested() ) );
	
	iUIManager->connectDeviceManager( iDevice );
	iUIManager->connectCommunicationManager( iCommunication );
}

void Engine::initCommunication()
{
	// initialize communication
	iCommunication = new CommunicationManager( *(iConfiguration->connectionSettings()) );
	connect( iCommunication, SIGNAL( error( int, CommunicationManager::CommunicationType  ) ),
			this, SLOT( errorHandler( int ) ) );
	connect( iCommunication, SIGNAL( meetingsFetched( const QList<Meeting*>& ) ),
			this, SLOT( meetingsFetched( const QList<Meeting*>& ) ) );
}

void Engine::initDevice()
{
	// create device manager
	iDevice = new DeviceManager( iConfiguration->startupSettings() );
	connect( iDevice, SIGNAL( error( int, const QString& ) ), this, SLOT( errorHandler( int, const QString& ) ) );	
	iDevice->initDeviceManager();
}

void Engine::dialogActivated()
{
	if ( iIdleTimeCounter != 0 )
	{
		iIdleTimeCounter->stop();
	}
}

void Engine::dialogDeactivated()
{
	if ( iIdleTimeCounter != 0 )
	{
		iIdleTimeCounter->start();
	}
}

void Engine::previousViewRestored()
{
	if ( iIdleTimeCounter != 0 )
	{
		iIdleTimeCounter->start();
	}
}

void Engine::stopIdleTimeCounter()
{
	if ( iIdleTimeCounter != 0 )
	{
		iIdleTimeCounter->stop();
	}
}

void Engine::startIdleTimeCounter()
{
	if ( iIdleTimeCounter != 0 )
	{
		iIdleTimeCounter->start();
	}
}

void Engine::currentRoomChanged(Room *aRoom)
{
	qDebug() << "[Engine::currentRoomChanged] <invoked>";
	iCurrentRoom = aRoom;
	roomStatusInfoNeeded( iCurrentRoom );
}
