#ifndef VIEWBASE_
#define VIEWBASE_

#include <QWidget>

class QEvent;
class QSize;

class ViewBase : public QWidget
{
	Q_OBJECT
	
public:
	enum ViewMode
	{
		NormalView, /*!< Indicates that the view is normal view that isn't hidden by events. */
		ObservedView /*!< Indicates that the view will be hidden when event occurs. */
	};
	
public:
	ViewBase( ViewBase::ViewMode aMode, QWidget *aParent = 0 );
	virtual ~ViewBase();
	
	//! Overwritten event handler.
	/*!
	 * Listens for events and emits eventDetected signal when observed
	 * event is triggered.
	 */
	virtual bool event(QEvent *event);
	
	//! Event filter.
	/*!
	 * Event filter method that is used to listen for child widgets
	 * events.
	 */
	virtual bool eventFilter(QObject *watched, QEvent *event);
	
	//! Returns view mode.
	/*!
	 * Returns the views mode which is one of the ViewMode enumerations.
	 */
	ViewBase::ViewMode viewMode();
	
public slots:
	//! This slot is called when the view is resized.
	/*!
	 * This slot will be called after the view is resized by the window manager
	 * to fit its client area. This method can be used to refine the view size
	 * after it has been resized, for example when the on screen keyboard is
	 * displayed.
	 */
	virtual void viewResized(const QSize &newSize, const QSize &oldSize) = 0;
	
	void connectionEstablished();
	
	void connectionLost();
	
signals:
	/*!
	 * This signal indicates that some user initiated event has occured.
	 * Event filter tracks for mouse, pointer and key events.
	 */
	void eventDetected();
	
protected:
	void observeChild(QWidget *aChild);
	bool connectionError; // true if last connection attempt failed
	// TODO: connectedOnce should actually indicate if the current week has been fetched at least once
	bool connectedOnce;	// true if connection has been formed at least once
	
private:
	ViewMode iViewMode;
};

#endif /*VIEWBASE_*/
